@@ -4,13 +4,15 @@
 //! Contains the slot map implementation.
 
 use std;
+#[cfg(feature = "unstable")]
+use std::collections::TryReserveError;
 use std::iter::{Enumerate, FusedIterator};
 use std::marker::PhantomData;
 use std::mem::ManuallyDrop;
 use std::ops::{Index, IndexMut};
 use std::{fmt, ptr};
 
-use super::{DefaultKey, Key, KeyData, Slottable};
+use crate::{DefaultKey, Key, KeyData, Slottable};
 
 // Storage inside a slot or metadata for the freelist when vacant.
 union SlotUnion<T: Slottable> {
@@ -266,6 +268,26 @@ impl<K: Key, V: Slottable> SlotMap<K, V> {
         self.slots.reserve(needed);
     }
 
+    /// Tries to reserve capacity for at least `additional` more elements to be
+    /// inserted in the `SlotMap`. The collection may reserve more space to
+    /// avoid frequent reallocations.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use slotmap::*;
+    /// let mut sm = SlotMap::new();
+    /// sm.insert("foo");
+    /// sm.try_reserve(32).unwrap();
+    /// assert!(sm.capacity() >= 33);
+    /// ```
+    #[cfg(feature = "unstable")]
+    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        // One slot is reserved for the sentinel.
+        let needed = (self.len() + additional).saturating_sub(self.slots.len() - 1);
+        self.slots.try_reserve(needed)
+    }
+
     /// Returns `true` if the slot map contains `key`.
     ///
     /// # Examples
@@ -1065,7 +1087,7 @@ mod serialize {
             D: Deserializer<'de>,
         {
             let mut slots: Vec<Slot<V>> = Deserialize::deserialize(deserializer)?;
-            if slots.len() >= (1 << 32) - 1 {
+            if slots.len() >= u32::max_value() as usize {
                 return Err(de::Error::custom(&"too many slots"));
             }
 
